//
//  FirstViewController.m
//  SIIMpaPrinterSample
//

#import "FirstViewController.h"
#import "PrinterManager.h"
#import "ValueManager.h"

@interface FirstViewController () <UITextFieldDelegate>

@property (weak, nonatomic) IBOutlet UIButton *listButton;
@property (weak, nonatomic) IBOutlet UIButton *callBackFunctionButton;
@property (weak, nonatomic) IBOutlet UIButton *sendFileButton;
@property (weak, nonatomic) IBOutlet UIButton *printLogoButton;
@property (weak, nonatomic) IBOutlet UIButton *printPhotoButton;
@property (weak, nonatomic) IBOutlet UIButton *printerInformationButton;
@property (weak, nonatomic) IBOutlet UIButton *barcodeButton;
@property (weak, nonatomic) IBOutlet UIButton *standardModeSampleButton;
@property (weak, nonatomic) IBOutlet UIButton *twoDCodeButton;
@property (weak, nonatomic) IBOutlet UIButton *pageModeSampleButton;
@property (weak, nonatomic) IBOutlet UIButton *textFormattingButton;
@property (weak, nonatomic) IBOutlet UITextField *addressTextField;
@property (weak, nonatomic) IBOutlet UISegmentedControl *portTypeSegment;

@end

@implementation FirstViewController {
    UIView *_indicatorView;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    [PrinterManager InitialSetting:self];
    [_addressTextField setDelegate:self];
    [self initUIButtons];
    [self initIndicatorView];
    [self setDefaultValue];
    [self updateConnectiontypeAndAddress];
}

#pragma mark - TextField
- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    [ValueManager setCurrentStringValue:KEY_TYPE_ADDRESS value:textField.text];
    return YES;
}

#pragma mark - Button Action
- (IBAction)portTypeSegmentValueChanged:(id)sender {
    if (_portTypeSegment.selectedSegmentIndex == SEGMENT_TCP_IP) {
        [ValueManager setCurrentStringValue:KEY_TYPE_PORTTYPE value:PORTTYPE_TCP_IP];
    } else if (_portTypeSegment.selectedSegmentIndex == SEGMENT_BLUETOOTH) {
        [ValueManager setCurrentStringValue:KEY_TYPE_PORTTYPE value:PORTTYPE_BLUETOOTH];
    }
}

- (IBAction)standardModeSampleButtonOnClicked:(id)sender {
    [self startLoadingView];
    dispatch_async(dispatch_queue_create("com.sii-ps.printStandardModeReceipt", DISPATCH_QUEUE_SERIAL), ^{
        BOOL success = [PrinterManager printStandardModeReceipt];
        [self stopLoadingView];
        if (success == NO) {
            [self showMessageBox:@"" message:[NSString stringWithFormat:@"%@%@", @"Attempt to print receipt failed.\n ErrorCode : ", [PrinterManager getErrorCodeText]]];
        }
    });
}

- (IBAction)pageModeSampleButtonOnClicked:(id)sender {
    [self startLoadingView];
    dispatch_async(dispatch_queue_create("com.sii-ps.printPageModeReceipt", DISPATCH_QUEUE_SERIAL), ^{
        BOOL success = [PrinterManager printPageModeReceipt];
        [self stopLoadingView];
        if (success == NO) {
            [self showMessageBox:@"" message:[NSString stringWithFormat:@"%@%@", @"Attempt to print receipt failed.\n ErrorCode : ", [PrinterManager getErrorCodeText]]];
        }
    });
}

- (IBAction)firstViewReturnActionForSegue:(UIStoryboardSegue *)segue {
    [self updateConnectiontypeAndAddress];
}

#pragma mark - Private
- (void)initUIButtons {
    [self configureUIButton:_listButton];
    [self configureUIButton:_callBackFunctionButton];
    [self configureUIButton:_sendFileButton];
    [self configureUIButton:_printLogoButton];
    [self configureUIButton:_printPhotoButton];
    [self configureUIButton:_printerInformationButton];
    [self configureUIButton:_barcodeButton];
    [self configureUIButton:_standardModeSampleButton];
    [_standardModeSampleButton setTitle:@"StandardMode\nSample" forState:UIControlStateNormal];
    [self configureUIButton:_twoDCodeButton];
    [self configureUIButton:_pageModeSampleButton];
    [_pageModeSampleButton setTitle:@"PageMode\nSample" forState:UIControlStateNormal];
    [self configureUIButton:_textFormattingButton];
}

- (void)configureUIButton:(UIButton *)_button {
    _button.layer.borderColor = [[UIColor colorWithRed:19/255 green:144/255 blue:255/255 alpha:1] CGColor];
    _button.layer.borderWidth = 1;
    _button.layer.cornerRadius = 5;
    _button.titleLabel.numberOfLines  = 0;
    _button.titleLabel.textAlignment = NSTextAlignmentCenter;
}

- (void)initIndicatorView {
    _indicatorView = [[NSBundle mainBundle] loadNibNamed:@"IndicatorView" owner:self options:nil][0];
    _indicatorView.frame = CGRectMake(0, 0, self.view.bounds.size.width, self.view.bounds.size.height);
    _indicatorView.hidden = YES;
    [self.view addSubview:_indicatorView];
}

- (void)setDefaultValue {
    if ([[ValueManager getCurrentStringValue:KEY_TYPE_PORTTYPE] length] == 0) {
        [[NSUserDefaults standardUserDefaults] setObject:@"address" forKey:KEY_TYPE_ADDRESS];
        [[NSUserDefaults standardUserDefaults] setObject:PORTTYPE_TCP_IP forKey:KEY_TYPE_PORTTYPE];
        [[NSUserDefaults standardUserDefaults] setObject:PRINTER_MODEL_MPA40 forKey:KEY_TYPE_PRINTERMODEL];
    }

    [[NSUserDefaults standardUserDefaults] setObject:SETTINGS_LOG_FILE_SIZE_50M forKey:KEY_TYPE_LOG_FILE_SIZE];
}

- (void)updateConnectiontypeAndAddress {
    _addressTextField.text = [ValueManager getCurrentStringValue:KEY_TYPE_ADDRESS];
    if ([[ValueManager getCurrentStringValue:KEY_TYPE_PORTTYPE] isEqualToString:PORTTYPE_TCP_IP]) {
        _portTypeSegment.selectedSegmentIndex = SEGMENT_TCP_IP;
    } else if ([[ValueManager getCurrentStringValue:KEY_TYPE_PORTTYPE] isEqualToString:PORTTYPE_BLUETOOTH]) {
        _portTypeSegment.selectedSegmentIndex = SEGMENT_BLUETOOTH;
    }
}

- (void)startLoadingView {
    dispatch_async(dispatch_get_main_queue(), ^{
        self->_indicatorView.hidden = NO;
    });
}

- (void)stopLoadingView {
    dispatch_async(dispatch_get_main_queue(), ^{
        self->_indicatorView.hidden = YES;
    });
}

- (void)showMessageBox:(NSString *)title message:(NSString *)message {
    dispatch_async(dispatch_get_main_queue(), ^{
        UIAlertController *alertController =  [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"Close" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        }]];
        [self presentViewController:alertController animated:YES completion:nil];
    });
}

@end
