//
//  PrintPhotoViewController.m
//  SIIMpaPrinterSample
//

#import "PrinterManager.h"
#import "PrintPhotoViewController.h"

#define DOCUMENTS_PATH [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0]

@interface PrintPhotoViewController () <UITableViewDelegate, UITableViewDataSource>

@property (weak, nonatomic) IBOutlet UITableView *tableView;

@end

@implementation PrintPhotoViewController {
    UIView *_indicatorView;
    NSArray *_tableData;
    NSInteger _currentValueIndex;    
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.tableView.delegate = (id <UITableViewDelegate>)self;
    self.tableView.dataSource = (id <UITableViewDataSource>)self;
    [self initFileList];
    [self initIndicatorView];
}

#pragma mark - Table view data source
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    if (section == 0) {
        return @"Select File";
    } else {
        return @"";
    }
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _tableData.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    cell.textLabel.text = [_tableData objectAtIndex:indexPath.row];
    if (indexPath.row == _currentValueIndex) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    for (NSInteger i = 0; i < _tableData.count; i++) {
        UITableViewCell *cell = [tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0]];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    cell.accessoryType = UITableViewCellAccessoryCheckmark;
    _currentValueIndex = indexPath.row;
}

#pragma mark - Button Action
- (IBAction)okButtonOnClicked:(id)sender {
    if (0 <= _currentValueIndex && _currentValueIndex < _tableData.count) {
        [self startLoadingView];
        dispatch_async([PrinterManager getDispatchQueue], ^{
            NSString *filePath = [NSString stringWithFormat:@"%@/%@", DOCUMENTS_PATH, self->_tableData[self->_currentValueIndex]];
            BOOL result = [PrinterManager printPhotoFile:filePath];
            [self stopLoadingView];
            if (result == NO) {
                [self showMessageBox:@"" message:[NSString stringWithFormat:@"%@%@", @"Attempt to print photo failed.\n ErrorCode : ", [PrinterManager getErrorCodeText]]];
            }
        });
    } else {
        [self dismissViewControllerAnimated:YES completion:nil];
    }
}

- (IBAction)closeButtonOnClicked:(id)sender {
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Private
- (void)initFileList {
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSArray *fileList = [[NSArray alloc] initWithArray:[fileManager contentsOfDirectoryAtPath:DOCUMENTS_PATH error:nil]];
    NSMutableArray *valueData = [[NSMutableArray alloc] init];
    if (fileList.count > 0) {
        for (int i = 0; i < fileList.count; i++) {
            NSString *fileName = [fileList objectAtIndex:i];
            NSString *filePath= [NSString stringWithFormat:@"%@/%@", DOCUMENTS_PATH, fileName];
            NSString *fileExtention = [filePath pathExtension];
            if ([fileExtention isEqualToString:@"bmp"] || [fileExtention isEqualToString:@"jpg"] || [fileExtention isEqualToString:@"jpeg"] || [fileExtention isEqualToString:@"png"]) {
                [valueData addObject:fileName];
            }
        }
    }
    _tableData = valueData;
    _currentValueIndex = -1;
    if (_tableData.count > 0) {
        _currentValueIndex = 0;
    }
    [self.tableView reloadData];
}

- (void)initIndicatorView {
    _indicatorView = [[NSBundle mainBundle] loadNibNamed:@"IndicatorView" owner:self options:nil][0];
    _indicatorView.frame = CGRectMake(0, 0, self.view.bounds.size.width, self.view.bounds.size.height);
    _indicatorView.hidden = YES;
    [self.view addSubview:_indicatorView];
}

- (void)startLoadingView {
    dispatch_async(dispatch_get_main_queue(), ^{
        self->_indicatorView.hidden = NO;
    });
}

- (void)stopLoadingView {
    dispatch_async(dispatch_get_main_queue(), ^{
        self->_indicatorView.hidden = YES;
    });
}

- (void)showMessageBox:(NSString *)title message:(NSString *)message {
    dispatch_async(dispatch_get_main_queue(), ^{
        UIAlertController *alertController =  [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
        [alertController addAction:[UIAlertAction actionWithTitle:@"Close" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        }]];
        [self presentViewController:alertController animated:YES completion:nil];
    });
}

@end
