//
//  PrinterListViewController.m
//  SIIMpaPrinterSample
//

#import "PrinterListViewController.h"
#import "PrinterManager.h"
#import "SIIMpaPrinterManager.h"
#import "ValueManager.h"

@interface PrinterListViewController () <UITableViewDelegate, UITableViewDataSource>

@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UIActivityIndicatorView *indicator;

@end

@implementation PrinterListViewController {
    NSArray *_tableData;
    NSInteger _currentValueIndex;
    dispatch_queue_t _gcd_printerDiscoverQueue;    
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.tableView.delegate = (id <UITableViewDelegate>)self;
    self.tableView.dataSource = (id <UITableViewDataSource>)self;
    [self serchPrinter];
}

#pragma mark - Table view data source
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
    if (section == 0) {
        return [[UIView alloc] initWithFrame:CGRectZero];
    } else {
        return nil;
    }
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    if (section == 0) {
        return 0.1;
    } else {
        return 0;
    }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _tableData.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    SIIMpaDiscoveredPrinter *printer = [_tableData objectAtIndex:indexPath.row];
    if (printer.ipAddress.length != 0) {
        cell.textLabel.text = [NSString stringWithFormat:@"%@(%@)", printer.printerModel, printer.ipAddress];
    } else if (printer.bluetoothAddress.length != 0) {
        cell.textLabel.text = [NSString stringWithFormat:@"%@(%@)", printer.printerModel, printer.bluetoothAddress];
    }
    if (indexPath.row == _currentValueIndex) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    for (NSInteger i = 0; i < _tableData.count; i++) {
        UITableViewCell *cell = [tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:0]];
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    cell.accessoryType = UITableViewCellAccessoryCheckmark;
    _currentValueIndex = indexPath.row;
}

#pragma mark - Button Action
- (IBAction)okButtonOnClicked:(id)sender {
    if (0 <= _currentValueIndex && _currentValueIndex < _tableData.count) {
        SIIMpaDiscoveredPrinter *printer = _tableData[_currentValueIndex];
        if (printer.ipAddress.length != 0) {
            [ValueManager setCurrentStringValue:KEY_TYPE_ADDRESS value:printer.ipAddress];
            [ValueManager setCurrentStringValue:KEY_TYPE_PORTTYPE value:PORTTYPE_TCP_IP];
        } else if (printer.bluetoothAddress.length != 0) {
            [ValueManager setCurrentStringValue:KEY_TYPE_ADDRESS value:printer.bluetoothAddress];
            [ValueManager setCurrentStringValue:KEY_TYPE_PORTTYPE value:PORTTYPE_BLUETOOTH];
        }
        [ValueManager setCurrentStringValue:KEY_TYPE_PRINTERMODEL value:printer.printerModel];
    }
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)closeButtonOnClicked:(id)sender {
    [PrinterManager cancelDiscoveryTcpPrinter];
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - Private
- (void)serchPrinter {
    [self startIndicator];
    NSMutableArray *printerList = [[NSMutableArray alloc] init];
    NSArray *btPrinter = [PrinterManager getBluetoothPrinterList];
    if (btPrinter.count > 0) {
        for (int i = 0; i < btPrinter.count; i++) {
            [printerList addObject:[btPrinter objectAtIndex:i]];
        }
    }
    _tableData = printerList;
    
    _currentValueIndex = -1;
    if (_tableData.count > 0) {
        _currentValueIndex = 0;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.tableView reloadData];
    });
    
    [PrinterManager startDiscoveryTcpPrinter];
    
    _gcd_printerDiscoverQueue = dispatch_queue_create("com.sii-ps.printerDiscover", DISPATCH_QUEUE_SERIAL);
    dispatch_async(_gcd_printerDiscoverQueue, ^{
        NSArray *beforePrinterList = self->_tableData.mutableCopy;
        while (true) {
            NSMutableArray *newPrinterList = beforePrinterList.mutableCopy;
            NSArray *lanPrinter = [PrinterManager getLanPrinterList];
            if (lanPrinter.count > 0) {
                for (int i = 0; i < lanPrinter.count; i++) {
                    [newPrinterList addObject:[lanPrinter objectAtIndex:i]];
                }
                self->_tableData = newPrinterList;
            }
            if (self->_currentValueIndex == -1 && self->_tableData.count > 0) {
                self->_currentValueIndex = 0;
            }
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.tableView reloadData];
            });
            if ([PrinterManager discoverFinished]) {
                break;
            }
            [NSThread sleepForTimeInterval:0.5f];
        }
        [self stopIndicator];
    });
}

- (void)startIndicator {
    dispatch_async(dispatch_get_main_queue(), ^{
        self->_indicator.hidden = NO;
    });
}

- (void)stopIndicator {
    dispatch_async(dispatch_get_main_queue(), ^{
        self->_indicator.hidden = YES;
    });
}

@end
